// Copyright (C) 1998 Ingo Ruhnke <grumbel@gmx.de>, see README.TXT for details

#include <iostream>
#include <ctime>
#include <cstdio>
#include <unistd.h>
#include <ClanLib/clanlib.h>

#include "playfield.hh"
#include "pingus.hh"
#include "pingu.hh"

char* 
PingusMain::get_title()
{
  return "Pingus";
}

void 
PingusMain::check_args(int argc, char* argv[])
{
  cout << "Checking arguments..." << endl; 
  int c;
  opterr = 0;
  while ((c = getopt (argc, argv, "l:")) != -1) {
    switch (c) {
    case 'l':
      strcpy(current_levelfile, optarg);
      current_level = -1;
      break;
    case '?':
      cout << endl << "Pingus" << endl;
      cout << "~~~~~~" << endl;
      cout << "-l FILE    Load a custom level from FILE" << endl;
      cout << endl;
      exit(EXIT_FAILURE);
      break;
    }
  }
    
  cout << "Arguments correct" << endl;
}

void
PingusMain::intro(void)
{
  int x = 0;
  int flips = 0;

  cout << "Pingus Intro..." << endl;

  CL_Surface* menu;
  CL_Surface* menu_quit;
  CL_Surface* menu_load;
  CL_Surface* menu_start;

  CL_Display::clear_display();
  menu->put_screen(0,0);
  CL_Display::flip_display();

  int  time = CL_System::get_time();
  int  pos = 0;
  int  temp_pos = pos;
 
  while(CL_Keyboard::get_keycode(CL_Keyboard::KEY_SPACE)==false) {
    CL_Display::flip_display();
    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_UP) && pos < 3) {
      ++pos;
    } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_DOWN) && pos > (-1)) {
      --pos;
    }
	
    if (temp_pos != pos) {
      cout << pos << endl;
      CL_Display::clear_display();
      cout << "Clearing Display...." << endl;
      menu->put_screen(0,0);
	
      switch(pos) {
      case 0:
	menu_quit->put_screen(45,246);
	break;

      case 1:
	menu_load->put_screen(45,300);

	break;
	
      case 2:
	menu_start->put_screen(45,192);
	break;
      }
      CL_Display::flip_display();
      temp_pos = pos;
    }

    //    if (time + 1000 < CL_System::get_time()) {
    cout << "Flipping Display..." << endl;
    time = CL_System::get_time();
    //    }

    if (CL_System::keep_alive()) break;  
  }
}

int
PingusMain::play_level(char* filename)
{
  bool quit = false;
  static int d=0;
  PLF* level_data = new PLF(filename);
  Playfield* playfield = new Playfield(level_data);

  //playfield->init(level_data);
  //  PinguLevel level = new PinguLevel(level_data);

  //  CL_Font* font = CL_Res_Font::load("Fonts/font", *resource);
      
  cout << "Starting Level" << endl;
  cout << "Playfield created" << endl;
      
  while(!quit) {
    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE) == true) {
      quit = true;
    }

    /* Rudimentary pause mechanism, not very use full at the moment
    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_P) == true) {
      while (CL_Keyboard::get_keycode(CL_Keyboard::KEY_Y) != true) {
	CL_System::keep_alive();
      }
    }*/

    CL_System::keep_alive();

    // Do the game
    playfield->let_move();

    // Draw the game
     if (d == 2) {
      d = 0;
      //CL_Display::clear_display();
      //font->print_left(50, 550, "BlaFish");
      playfield->draw();
      CL_Display::flip_display();
    }
    ++d;
  }  
}

int
PingusMain::play_level(int level_number)
{
  if (current_level != -1) {
    char* level_name;
    
    level_name = new char[255];  // dirty, should use string instead
    
    sprintf(level_name, "../levels/level%d.plf", level_number);
    cout << level_name << endl;
    
    play_level(level_name);

    delete[] level_name;
  } else {
    play_level(current_levelfile);
  }
}

void
PingusMain::load_data()
{
  resource = new CL_ResourceManager(pingus_datafile);

  Pingu::load_data(resource);
  World::load_data(resource);
  Playfield::load_data(resource);
}
  
int
PingusMain::main(int argc, char** argv)
{
  cout << "Pingus " << VERSION << " starting..." << endl;

  PingusMain::check_args(argc, argv);

  pingus_data_dir = getenv("PINGUS_DATA_DIR");

  if (!pingus_data_dir) {
    cout << "Please set PINGUS_DATA_DIR to the position of the \"pingus.scr\" file." << endl;
    cout << "Example: PINGUS_DATA_DIR=\"/usr/src/pingus/scr/\"" << endl;
    exit(EXIT_FAILURE);
  }

  pingus_datafile = new char[255];
  
  cout << "Pingus Data File: " << pingus_datafile << endl;
  strcpy(pingus_datafile, pingus_data_dir);
  strcat(pingus_datafile, "pingus.scr");
  cout << "Pingus Data File: " << pingus_datafile << endl;

  // initialisation
  CL_System::init_display();
  CL_Display::set_videomode(800, 600, 16);
  
  //  datafile = CL_InputSourceProvider::create_datafile_provider("pingus.dat");

  cout << "Loading Data file..." << endl;
  //  cout << "Starting: loading sprite..." << Pingu::sprite << endl;

  load_data();

  // cout << "Finished: loading sprite... " << Pingu::sprite << endl;

  cout << "finished" << endl;

  int scr_width = CL_Display::get_width();
  int scr_height = CL_Display::get_height();

  //intro();

  cout << "Starting main loop" << endl;

  int x=0, y=0;
  
  //      while (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE) == false) {
  //    cout << CL_Input::mices[0]->get_mouse_x() << "--" 
  //	 << CL_Input::mices[0]->get_mouse_y() << endl;
  //stopper->put_screen(x,y);  
  srand(time(NULL));
  //ame();
  play_level(2);
  cout << "Looping" << endl;
  //	CL_System::keep_alive(); // Do NOT forget this one...
  // }
  return 0;
}

/* EOF */
